(* Title:  BaseDefinitions/eval-def.sml
   Author: Walther Neuper
   (c) due to copyright terms

Here is a minimum of code required for Know_Store.thy.
For main code see structure Eval.
*)
signature EVALUATE__DEFINITION =
sig
  eqtype const_id
  type ml_fun = Rule_Def.eval_fn
  val ml_fun_empty: ml_fun
  type ml

  eqtype prog_id
  type ml_from_prog

  val equal: ml_from_prog * ml_from_prog -> bool

  val ml_fun_to_store: ml_fun -> theory -> theory
  val ml_fun_from_store: theory -> ml_fun

end

(**)
structure Eval_Def(**): EVALUATE__DEFINITION(**) =
struct
(**)

type const_id = Rule_Def.eval_const_id (* op in Isabelle term "Const( op, _)" *)
type ml_fun = Rule_Def.eval_fn
fun ml_fun_empty (_ : 'a) (_ : term) (_ : Proof.context) = NONE : (string * term) option;
type ml = const_id * ml_fun;

type prog_id = string; (* accepted by partial_function in method *)
type ml_from_prog =  (* fetched by fun calculate_                   *)
  prog_id *          (* a simple identifier used in programs        *)
  (const_id *        (* a long identifier used in Const             *)
    ml_fun)          (* ML function for evaluation within rewriting *)

fun equal ((pi1, (ci1, _)), (pi2, (ci2, _))) =
  if pi1 = pi2
  then if ci1 = ci2 then true else raise ERROR ("equal: " ^ ci1 ^ " <> " ^ ci2)
  else false


(* theory data for Isar command 'calculation' *)

structure Data = Theory_Data
(
  type T = ml_fun option;
  val empty = NONE;
  val extend = I;
  fun merge _ = NONE;
);

val ml_fun_to_store = Data.put o SOME;
val ml_fun_from_store = the o Data.get;

end
