(* Title:  Interpret/lucas-interpreter.sml
   Author: Walther Neuper 2019
   (c) due to copyright terms

\<open>ML_structure Problem\<close> holds the essential information required for automated doing
\<open>ML_structure Specification\<close> and \<open>ML_structure Solution\<close>.

Why \<open>ML_structure Problem\<close> is not parsed on the fly within the current \<open>ML_structure Context\<close> 
see \<open>ML_structure Refine\<close>.
*)

signature METHOD =
sig
  type T = Meth_Def.T
  val empty: T

  type id = Meth_Def.id
  val id_empty: id
  val id_to_string: id -> string

  type input
  (* TODO: ------------ remove always empty --- vvvvvvvvvvv -- vvv*)
  val prep_input: theory ->  Check_Unique.id -> string list -> id ->
     id * Problem.model_input * input * thm -> T * id

  val set_data: input -> theory -> theory
  val the_data: theory -> input

  val from_store': theory -> id -> T
  val from_store: Proof.context -> id -> T
end

(**)
structure MethodC(**): METHOD(**) =
struct
(**)

type T = Meth_Def.T;
val empty = Meth_Def.empty;

type id = Meth_Def.id;
val id_empty = Meth_Def.id_empty;
val id_to_string = Meth_Def.id_to_string;


(** prepare MethodC for Store **)

(* a subset of MethodC.T record's fields *)
type input = 
  {calc: Rule_Def.eval_ml_from_prog list, crls: Rule_Set.T, errpats: Error_Pattern_Def.T list, nrls: Rule_Set.T,
    prls: Rule_Set.T, rew_ord': Rewrite_Ord.id, rls': Rule_Set.T, srls: Rule_Set.T}

fun prep_input thy guh (mathauthors: string list) (init: References_Def.id)
	    (metID, ppc,
        {rew_ord' = ro, rls' = rls, srls = srls, prls = prls, calc = _(*scr_isa_fns*), crls = cr,
          errpats = ep, nrls = nr}, scr) =
    let
      fun eq f (f', _) = f = f';
      val gi = filter (eq "#Given") ppc;
      val gi = (case gi of
		    [] => (writeln ("prep_input: in " ^ guh ^ " #Given is empty ?!?"); [])
		  | ((_, gi') :: []) => 
        (*(case \<^try>\<open> *)map (Problem.split_did o (Syntax.read_term_global thy)) gi'(*\<close> of
          SOME x => x
        | NONE => raise ERROR ("prep_pbt: syntax raise ERROR in '#Given' of " ^ strs2str metID))*)
		  | _ => raise ERROR ("prep_pbt: more than one '#Given' in " ^ strs2str metID));
		  val gi = map (pair "#Given") gi;

		  val fi = filter (eq "#Find") ppc;
		  val fi = (case fi of
		    [] => (writeln ("prep_input: in " ^ guh ^ " #Find is empty ?!?"); [])
		  | ((_, fi') :: []) => 
        (*(case \<^try>\<open> *)map (Problem.split_did o (Syntax.read_term_global thy)) fi'(*\<close> of
          SOME x => x
        | NONE => raise ERROR ("prep_pbt: syntax raise ERROR in '#Find' of " ^ strs2str metID))*)
		  | _ => raise ERROR ("prep_pbt: more than one '#Find' in " ^ strs2str metID));
		  val fi = map (pair "#Find") fi;

		  val re = filter (eq "#Relate") ppc;
		  val re = (case re of
		    [] => (writeln ("prep_input: in " ^ guh ^ " #Relate is empty ?!?"); [])
		  | ((_,re') :: []) =>
        (*(case \<^try>\<open> *) map (Problem.split_did o (Syntax.read_term_global thy)) re'(*\<close> of
          SOME x => x
        | NONE => raise ERROR ("prep_pbt: syntax raise ERROR in '#Relate' of " ^ strs2str metID))*)
		  | _ => raise ERROR ("prep_pbt: more than one '#Relate' in " ^ strs2str metID));
		  val re = map (pair "#Relate") re;

		  val wh = filter (eq "#Where") ppc;
		  val wh = (case wh of
		    [] => (writeln ("prep_input: in " ^ guh ^ " #Where is empty ?!?"); [])
		  | ((_, wh') :: []) => 
        (*(case \<^try>\<open> *)map (TermC.parse_patt_PIDE thy) wh'(*\<close> of
          SOME x => x
        | NONE => raise ERROR ("prep_pbt: syntax raise ERROR in '#Where' of " ^ strs2str metID))*)
		  | _ => raise ERROR ("prep_pbt: more than one '#Where' in " ^ strs2str metID));
		  val sc = Program.prep_program scr
		  val calc = if Thm.eq_thm (scr, @{thm refl}) then [] else Auto_Prog.get_calcs thy sc
    in
       ({guh = guh, mathauthors = mathauthors, init = init, ppc = gi @ fi @ re, pre = wh,
         rew_ord' = ro, erls = rls, srls = srls, prls = prls, calc = calc,
         crls = cr, errpats = ep, nrls = nr, scr = Rule.Prog sc}, metID)
    end;


(** Isar command **)

structure Data = Theory_Data
(
  type T = input option;
  val empty = NONE;
  val extend = I;
  fun merge _ = NONE;
);

val set_data = Data.put o SOME;
val the_data = the o Data.get;

local

val parse_program =
  Scan.option (Problem.parse_item \<^keyword>\<open>Program\<close> (Parse.position Parse.name));

val ml = ML_Lex.read;

val _ =
  Outer_Syntax.command \<^command_keyword>\<open>method\<close>
    "prepare ISAC method and register it to Knowledge Store"
    (Parse.name -- Parse.!!! (\<^keyword>\<open>:\<close> |-- Parse.name --
      Parse.!!! (\<^keyword>\<open>=\<close> |-- Parse.ML_source -- Problem.parse_authors --
        parse_program -- Problem.parse_model_input)) >>
      (fn (name, (id, (((source, maa), program), model_input))) => Toplevel.theory (fn thy =>
        let
          val metID = References_Def.explode_id id;
          val set_data =
            ML_Context.expression (Input.pos_of source)
              (ml "Theory.setup (MethodC.set_data (" @ ML_Lex.read_source source @ ml "))")
            |> Context.theory_map;
          val input = the_data (set_data thy);
          val thm =
            (case program of
              NONE => @{thm refl}
            | SOME a => Global_Theory.get_thm thy (Global_Theory.check_fact thy a));
          val arg = prep_input thy name maa id_empty (metID, model_input, input, thm);
        in KEStore_Elems.add_mets @{context} [arg] thy end)))

in end;

(** get MethodC from Store **)

fun adapt_to_type ctxt {guh, mathauthors, init, rew_ord', erls, srls, crls, nrls, errpats, calc, 
    scr, prls, ppc, pre} =
  let
    val where_ = map (Model_Pattern.adapt_term_to_type ctxt) pre
    val model = map (Model_Pattern.adapt_to_type ctxt) ppc
    val errpats = map (Error_Pattern_Def.adapt_to_type ctxt) errpats
  in
    {guh = guh, mathauthors = mathauthors, init = init, rew_ord' = rew_ord', erls = erls, 
    srls = srls, crls = crls, nrls = nrls, errpats = errpats, calc = calc, scr = scr, prls = prls, 
    ppc = model, pre = where_}
    end

(* TODO: throws exn 'get_pbt not found: ' ... confusing !! take 'ketype' as an argument *)
fun from_store' thy id =
  (Store.get (KEStore_Elems.get_mets thy) id id)
    handle ERROR _ =>
      raise error ("*** ERROR MethodC.from_store thy " ^ strs2str id ^ " not found");
fun from_store ctxt id =
  let
    val pbl = Store.get (get_mets ()) id id
  in adapt_to_type ctxt pbl end
    handle ERROR _ =>
      raise error ("*** ERROR MethodC.from_store thy " ^ strs2str id ^ " not found");

(**)end(**)
